@@ -7,8 +7,11 @@ use Time::Local;
 use NBU;
 
 my %opts;
-getopts('utbseidjmfc:C:a:p:n:', \%opts);
+getopts('dutbseiDjmfc:C:a:p:n:l:o:', \%opts);
 
+#
+# The -a <yyyymmdd> option controls the starting date for the log file analysis
+# process.  If absent, today's date is assumed to be of interest
 my $period = 1;
 my $mm;  my $dd;  my $yyyy;
 if (!$opts{'a'}) {
@@ -29,6 +32,10 @@ else {
   $yyyy = $1;
 }
 
+#
+# The parseTime function and its static companions $lastParsedTime and
+# $midnight are used to translate the time-stamps from the input log file into
+# un*x timestamps
 my $lastParsedTime;
 my $midnight;
 sub parseTime {
@@ -46,6 +53,9 @@ sub parseTime {
   return $lastParsedTime = $tm;
 }
 
+#
+# Delta un*x timestamps are converted into human readable format by this
+# function, dispInterval
 sub dispInterval {
   my $i = shift;
 
@@ -60,12 +70,40 @@ sub dispInterval {
   return $fmt;
 }
 
+#
+# Symantec (formerly Veritas) succumbed to changing the log file names based
+# on the host OS.  The -o option allows one to specify which OS's logfiles
+# we are dealing with and thus switch name patterns
+my $logFilePattern = "log.%s";
+if ($opts{'o'} =~ /indow/) {
+  $logFilePattern = "%s.log";
+}
+
+#
+# Without further reference to a location for the log files we assume the
+# standard lcoation for the OS type.  The log file directory can be overriden
+# with the -l option.
 my $logPath = "/opt/openv/netbackup/logs";
+my $bptmLogPath = $logPath."/bptm";
+if ($opts{'l'}) {
+  $bptmLogPath = $opts{'l'};
+}
+
+my @monthDays = ( 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
+
 my $asOf;
 my $daysLeft = $period;
 while ($daysLeft--) {
   my $mmddyy = sprintf("%02d%02d%02d", $mm, $dd, ($yyyy % 100));
-  open (LOG, "<${logPath}/bptm/log.".$mmddyy);
+  my $logFile = $bptmLogPath.sprintf($logFilePattern, $mmddyy);
+
+  if (!open(LOG, "<$logFile")) {
+    print STDERR "Could not open $logFile\n";
+    exit;
+  }
+  if ($opts{'d'}) {
+    print STDERR "Processing data for $yyyy$mm$dd\n";
+  }
 
   $midnight = timelocal(0, 0, 0, $dd, $mm-1, $yyyy);
   $lastParsedTime = undef;
@@ -73,13 +111,11 @@ while ($daysLeft--) {
 
   while (<LOG>) {
     #
-    if (/\[([\d]+)\] <2> bptm: INITIATING: -([\S]+)/) {
+    if (/\[([\d\.]+)\] <2> bptm: INITIATING(|[\s]*\(.+\)): -([\S]+)/) {
       my $pid = $1;
-      my $option = $2;
-
-      my $job = NBU::Job->new($pid);
-
-      $job->start(parseTime($_));
+      my $flags = $2;
+      my $option = $3;
+      my $job;
 
       #
       # "w" jobs are your basic archive operations
@@ -87,39 +123,139 @@ while ($daysLeft--) {
       # an image (which it therefore promptly returns) and the remaining
       # attributes are part of the image, not the job 
       if ($option eq "w") {
-        /-jobid ([\d]+) /;  $job->id($1);
-        /-stunit ([\S]+) /;  $job->storageUnit($1);
-        /-mediasvr ([\S]+) /;  $job->mediaServer(NBU::Host->new($1));
-        /-b ([\S]+) /;  my $image = $job->image($1);
-        /-cl ([\S]+) /;  $image->class(NBU::Class->new($1));
-        /-c ([\S]+) /;  $image->client(NBU::Host->new($1));
+        /-jobid ([\d]+) /;  {
+	  my $id = $1;
+
+	  #
+	  # Check to see if a job with this id already exists; then we might
+	  # be the child (reader) process and no new job needs be created.
+	  if (my $parent = NBU::Job->byID($id)) {
+	    if ($parent->readerPID eq $pid) {
+	      next;
+	    }
+	    else {
+	      $id = $id.".1";
+	    }
+	  }
+	  $job = NBU::Job->new($pid);
+	  $job->start(parseTime($_));
+	  $job->id($id);
+          /-stunit ([\S]+) /;  $job->storageUnit($1);
+          /-mediasvr ([\S]+) /;  $job->mediaServer(NBU::Host->new($1));
+          /-b ([\S]+) /;  my $image = $job->image($1);
+          /-cl ([\S]+) /;  $image->class(NBU::Class->new($1));
+          /-c ([\S]+) /;  $image->client(NBU::Host->new($1));
+	}
       }
       # when handed a "pid" we will likely be involved in a restore
       elsif ($option eq "pid") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
       }
       # deleting images
       elsif ($option eq "d") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
       }
       # some jobs operate on a single media
       elsif ($option eq "ev") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
+      }
+      elsif ($option eq "count") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
+      }
+      elsif ($option eq "countmedia") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
+      }
+      elsif ($option eq "delete_expired") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
+      }
+      elsif ($option eq "U") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
+      }
+      elsif ($option eq "load") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
+      }
+      elsif ($option eq "unload") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
+      }
+      elsif ($option eq "change_exp_date") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
+      }
+      elsif ($option eq "mlist") {
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
+      }
+      else {
+print STDERR "Unknown job option $option\n";
+	$job = NBU::Job->new($pid);
+	$job->start(parseTime($_));
       }
     }
-    if (/\[([\d]+)\] <2> select_media: selected media id ([\S]+) for backup/) {
+
+    if (/\[([\d\.]+)\] <2> write_backup: backup child process is pid ([\d\.]+)/) {
       my $job = NBU::Job->byPID($1);
+      next if (!defined($job));
+
+      $job->readerPID($2);
+    }
+
+    if (/\[([\d\.]+)\] <2> select_media: selected media id ([\S]+) for backup/) {
+      my $job = NBU::Job->byPID($1);
+
+      next if (!defined($job));
+
       my $volume = NBU::Media->byID($2);
 
       $job->startMounting(parseTime($_), $volume);
     }
-    if (/\[([\d]+)\] <2> write_backup: media id ([\S]+) mounted on drive index ([\d]+)/) {
+    if (/\[([\d\.]+)\] <2> write_backup: media id ([\S]+) mounted on drive index ([\d]+)/) {
       my $job = NBU::Job->byPID($1);
+
+      next if (!defined($job));
+
       my $mediaID = $2;
       my $drive = NBU::Drive->byIndex($3, $job->mediaServer);
 
       $job->mounted(parseTime($_), $drive);
     }
-    if (/\[([\d]+)\] <4> write_backup: successfully wrote/) {
+
+    if (/\[([\d\.]+)\] <2> io_init: using ([\d]+) data buffer size/) {
       my $job = NBU::Job->byPID($1);
 
+      if (!defined($job)) {
+        print STDERR "No job initiated by PID $1?\n";
+        next
+      }
+
+      $job->dataBufferSize($2);
+    }
+    if (/\[([\d\.]+)\] <2> io_init: using ([\d]+) data buffers/) {
+      my $job = NBU::Job->byPID($1);
+
+      next if (!defined($job));
+      $job->dataBufferCount($2);
+    }
+    if (/\[([\d\.]+)\] <2> io_init: child delay = ([\d]+), parent delay = ([\d]+) \((milliseconds)\)/) {
+      my $job = NBU::Job->byPID($1);
+
+      next if (!defined($job));
+      $job->delayCycles($1, $2);
+    }
+
+    if (/\[([\d\.]+)\] <4> write_backup: successfully wrote/) {
+      my $job = NBU::Job->byPID($1);
+
+      next if (!defined($job));
+
       /fragment ([\d]+), ([\d]+) Kbytes at ([\d]+\.[\d]+)/;
       my $fragment = $1;
       my $size = $2;
@@ -127,21 +263,28 @@ while ($daysLeft--) {
 
       $job->write($fragment, $size, $speed);
     }
-    if (/\[([\d]+)\] <2> fill_buffer: \[([\d]+)\] socket is closed, waited for empty buffer ([\d]+) times, delayed ([\d]+) times, read ([\d]+) bytes/) {
-      my $job = NBU::Job->byPID($2);
-
-      $job->ioStats($3, $4, $5);
-    }
-    if (/\[([\d]+)\] <2> write_data: waited for full buffer ([\d]+) times, delayed ([\d]+) times/) {
+    if (/\[([\d\.]+)\] <2> fill_buffer: \[([\d]+)\] socket is closed, waited for empty buffer ([\d]+) times, delayed ([\d]+) times, read ([\d]+) bytes/) {
       my $job = NBU::Job->byPID($1);
-      $job->ioStats($2, $3);
+
+      if (!defined($job)) {
+	print STDERR "No job for fill_buffer stats, PID: $1\n";
+	next;
+      }
+      $job->networkReadStats($3, $4);
     }
-    if (/\[([\d]+)\] <2> .* tpunmount.ing .*tpreq\/([\S]+)/) {
+    if (/\[([\d\.]+)\] <2> write_data: waited for full buffer ([\d]+) times, delayed ([\d]+) times/) {
+
+      my $job = NBU::Job->byPID($1);
+      next if (!defined($job));
+
+      $job->driveWriteStats($2, $3);
+    }
+    if (/\[([\d\.]+)\] <2> .* tpunmount.ing .*tpreq\/([\S]+)/) {
       if (my $media = NBU::Media->byID($2)) {
         $media->unmount(parseTime($_));
       }
     }
-    if (/\[([\d]+)\] <2> (bptm|catch_signal|mpx_terminate_exit): EXITING with status ([\d]+)/) {
+    if (/\[([\d\.]+)\] <2> (bptm|catch_signal|mpx_terminate_exit|mm_terminate): EXITING with status ([\d]+)/) {
       my $job = NBU::Job->byPID($1);
 
       if (!$job) {
@@ -161,6 +304,10 @@ print STDERR "EXITING non existent job from pid \"$1\"\n";
     }
   }
   $dd += 1;
+  if ($monthDays[$mm] < $dd) {
+    $mm += 1;
+    $dd = 1;
+  }
 }
 
 my @list = NBU::Job->list;
@@ -177,6 +324,7 @@ foreach $j (@list) {
 
 if (!$j->id) {
     print STDERR "Process was not eliminated? ".$j->pid."\n";
+    next;
 }
 
   if ($opts{'e'}) {
@@ -198,14 +346,14 @@ if (!$j->id) {
 
   my $jid = $j->id;
   if ($opts{'j'}) {
-    my $tmb = sprintf("%.2f", $j->dataWritten / 1024);  my $tunits = "Mb";
+    my $tmb = sprintf("%.2f", $j->dataWritten / 1024);  my $tunits = "MB";
     if ($tmb > 1024) {
-      $tmb = sprintf("%.2f", $j->dataWritten / 1024 / 1024);  $tunits = "Gb";
+      $tmb = sprintf("%.2f", $j->dataWritten / 1024 / 1024);  $tunits = "GB";
     }
     my $sp = sprintf("%.2f", ($j->dataWritten / $j->elapsedTime / 1024));
-    print "J:$jid".":".sprintf("%5u", $j->pid).":".$j->class->name." ".localtime($j->start);
+    print "J:$jid".":".sprintf("%5s", $j->pid).":".$j->class->name." ".localtime($j->start);
     if (defined($j->status)) {
-      print " ${tmb}${tunits} in ".dispInterval($j->elapsedTime)." (${sp}Mb/s)";
+      print " ${tmb}${tunits} in ".dispInterval($j->elapsedTime)." (${sp}MB/s)";
       print " status ".$j->status."\n";
     }
     else {
@@ -233,9 +381,9 @@ if (!$j->id) {
 
     if ($opts{'m'}) {
       my $wt = sprintf("%.2f", $mount->writeTime);
-      my $mb = sprintf("%.2f", ($mount->dataWritten / 1024));  my $units = "Mb";
+      my $mb = sprintf("%.2f", ($mount->dataWritten / 1024));  my $units = "MB";
       if ($mb > 1024) {
-        $mb = sprintf("%.2f", ($mount->dataWritten / 1024 / 1024));  $units = "Gb";
+        $mb = sprintf("%.2f", ($mount->dataWritten / 1024 / 1024));  $units = "GB";
       }
       my $sp = 0;
       if ($mount->writeTime) {
@@ -243,19 +391,29 @@ if (!$j->id) {
       }
       print "M:${jid}:${mediaID}".sprintf(" in %2u ", $mount->drive->id).localtime($mount->start).
             " ${mb}${units} over ".dispInterval($wt).
-            " at ${sp}Mb/s\n";
+            " at ${sp}MB/s\n";
     }
 
     $totalWriteTime += $mount->writeTime;
   }
 
   if ($opts{'j'}) {
-    if ($opts{'s'}) {
-      my $overHeadTime = sprintf("%.2f", ($j->elapsedTime - $totalWriteTime));
-      my $overHead = sprintf("%.2f", ($overHeadTime * 100)/ $j->elapsedTime);
-      my ($noBuffer, $noData) = $j->ioStats;
-      print "J:${jid}:ended ".localtime($j->stop)." overhead ${overHeadTime}s ($overHead\%)".
-              " $noBuffer/$noData\n";
+    if ($opts{'s'} && defined($j->status)) {
+      my $overHeadLength = sprintf("%.2f", ($j->elapsedTime - $totalWriteTime));
+      my $overHead = sprintf("%.2f", ($overHeadLength * 100)/ $j->elapsedTime);
+      my $overHeadTime = dispInterval(sprintf("%u", $overHeadLength));
+
+      my ($noFullBuffer, $fullDelayCount, $fullDelay) = $j->driveWriteStats();
+      my ($noEmptyBuffer, $emptyDelayCount, $emptyDelay) = $j->networkReadStats();
+      if ($emptyDelayCount && $totalWriteTime) {
+	my $overHead = sprintf("%.2f", ($emptyDelay * 100)/ $totalWriteTime);
+	print "J:${jid}:delays reading $emptyDelayCount cycles for ".dispInterval($emptyDelay)." or $overHead\%\n"
+      }
+      if ($fullDelayCount && $totalWriteTime) {
+	my $overHead = sprintf("%.2f", ($fullDelay * 100)/ $totalWriteTime);
+	print "J:${jid}:delays writing $fullDelayCount cycles for ".dispInterval($fullDelay)." or $overHead\%\n"
+      }
+      print "J:${jid}:ended ".localtime($j->stop)." overhead ${overHeadTime} ($overHead\%)\n";
     }
   }
   $lastJob = $j;
@@ -264,17 +422,17 @@ if (!$j->id) {
   $overallElapsedTime += $j->elapsedTime;
 }
 
-if (!$opts{'d'} && !$opts{'u'}) {
+if (!$opts{'D'} && !$opts{'u'}) {
   my $overallVolumesUsed = (keys %volumesUsed);
   $overallDataWritten = sprintf("%.2f", ($overallDataWritten / 1024 / 1024));
-  print "$jobCounter jobs wrote ${overallDataWritten}Gb over ".
+  print "$jobCounter jobs wrote ${overallDataWritten}GB over ".
     dispInterval($overallElapsedTime).
     " to $overallVolumesUsed distinct volumes\n";
 }
 
 #print "Jobs ran from ".localtime($firstJob->start)." until ".localtime($lastJob->stop)."\n";
 
-if ($opts{'d'}) {
+if ($opts{'D'}) {
   my @dl = NBU::Drive->pool;
 
   @dl = sort { $a->id <=> $b->id} (@dl);
@@ -283,6 +441,7 @@ if ($opts{'d'}) {
     foreach my $d (@dl) {
       my $header = "Drive ".$d->id."\n";
       my $usage = $d->usage;
+      @$usage = (sort {$$a{START} <=> $$b{START} } @$usage);
       foreach my $use (@$usage) {
 
         my $mount = $$use{'MOUNT'};
@@ -295,10 +454,10 @@ if ($opts{'d'}) {
         print $header;  $header = "";
         my $mediaID = $mount->volume->id;
         if ($$use{'STOP'}) {
-          my $u = "Mb";
+          my $u = "MB";
           my $dw = sprintf("%.2f", ($mount->dataWritten / 1024));
           if ($dw > 1024) {
-            $dw = sprintf("%.2f", $mount->dataWritten / 1024 / 1024);  $u = "Gb";
+            $dw = sprintf("%.2f", $mount->dataWritten / 1024 / 1024);  $u = "GB";
           }
           my $sp = "-.--";
           if ($mount->writeTime) {
@@ -306,7 +465,7 @@ if ($opts{'d'}) {
           }
           print localtime($$use{'START'})." ${dw}${u} over ".
                 dispInterval($mount->writeTime).
-                " at ${sp}Mb/s onto $mediaID";
+                " at ${sp}MB/s onto $mediaID";
         }
         else {
           print localtime($$use{'START'})." still running onto $mediaID";
@@ -360,6 +519,7 @@ if ($opts{'u'}) {
   foreach my $d (@dl) {
     my $id = $d->id;
     my $usage = $d->usage;
+    @$usage = (sort {$$a{START} <=> $$b{START} } @$usage);
 
     my $step = $asOf;
     my $use = shift @$usage;
