@@ -6,8 +6,7 @@ use Getopt::Std;
 use NBU;
 
 my %opts;
-getopts('csd', \%opts);
-$opts{'c'} = 1;
+getopts('ed', \%opts);
 
 NBU::->debug($opts{'d'});
 
@@ -19,6 +18,7 @@ foreach my $l (@ll) {
   print "License key ".$l->key."\n";
     print "  Product ".$l->productDescription($l->product)."\n";
     print "  Server Tier ".$l->serverTier."\n";
+    print "  Expires: ".(defined($l->expiration) ? localtime($l->expiration) : "Never")."\n";
     my @fcl = $l->features;
     foreach my $fc (@fcl) {
       print "    Feature ".$l->featureDescription($fc)."\n";
