@@ -17,7 +17,7 @@ BEGIN {
   use Exporter   ();
   use AutoLoader qw(AUTOLOAD);
   use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $AUTOLOAD);
-  $VERSION =	 do { my @r=(q$Revision: 1.22 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r };
+  $VERSION =	 do { my @r=(q$Revision: 1.24 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r };
   @ISA =         qw();
   @EXPORT =      qw();
   @EXPORT_OK =   qw();
@@ -77,6 +77,7 @@ my %classTypes = (
   20 => "FlashBackup",
   21 => "SplitMirror",
   22 => "AFS",
+  30 => "Vault",
 );
 
 my $rollCalled = 0;
@@ -160,7 +161,7 @@ sub loadClasses {
     }
     if (/^RES /) {
       my ($tag, @residences) = split;
-      $class->{RESIDENCE} = $residences[0] unless ($residences[0] eq "*NULL*");
+      $class->{RESIDENCE} = NBU::StorageUnit->byLabel($residences[0]) unless ($residences[0] eq "*NULL*");
       next;
     }
     if (/^POOL /) {
@@ -338,6 +339,12 @@ sub residence {
   return $self->{RESIDENCE};
 }
 
+sub storageUnit {
+  my $self = shift;
+
+  return $self->residence(@_);
+}
+
 my %policyAware = (
   4 => "Oracle",
   6 => "Informix",
@@ -361,6 +368,8 @@ sub type {
   if (@_) {
     $self->{TYPE} = shift;
   }
+  print STDERR "Asked to return unknown type ".$self->{TYPE}." for ".$self->{NAME}."\n"
+   if !defined($classTypes{$self->{TYPE}});
   return $classTypes{$self->{TYPE}};
 }
 
